@@ -47,8 +47,7 @@ template '__jifty/subs' => sub {
     $writer->xmlDecl( "UTF-8", "yes" );
 
     my $begin = <<'END';
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
- "http://www.w3.org/TR/2002/REC-xhtml1-20020801/DTD/xhtml1-strict.dtd">
+<!DOCTYPE html>
 <html><head><title></title></head>
 END
     chomp $begin;
