@@ -2,7 +2,7 @@
 Copyright (c) 2007, Yahoo! Inc. All rights reserved.
 Code licensed under the BSD License:
 http://developer.yahoo.net/yui/license.txt
-version: 2.2.1
+version: 2.4.1
 */
 /**
  * Provides Attribute configurations.
@@ -211,7 +211,7 @@ YAHOO.util.Attribute.prototype = {
      * @uses YAHOO.util.EventProvider
      */
     YAHOO.util.AttributeProvider = function() {};
-    
+
     YAHOO.util.AttributeProvider.prototype = {
         
         /**
@@ -228,8 +228,8 @@ YAHOO.util.Attribute.prototype = {
          * @param {String} key The attribute whose value will be returned.
          */
         get: function(key){
-            var configs = this._configs || {};
-            var config = configs[key];
+            this._configs = this._configs || {};
+            var config = this._configs[key];
             
             if (!config) {
                 return undefined;
@@ -247,8 +247,8 @@ YAHOO.util.Attribute.prototype = {
          * @return {Boolean} Whether or not the value was set.
          */
         set: function(key, value, silent){
-            var configs = this._configs || {};
-            var config = configs[key];
+            this._configs = this._configs || {};
+            var config = this._configs[key];
             
             if (!config) {
                 return false;
@@ -263,12 +263,12 @@ YAHOO.util.Attribute.prototype = {
          * @return {Array} An array of attribute names.
          */
         getAttributeKeys: function(){
-            var configs = this._configs;
+            this._configs = this._configs;
             var keys = [];
             var config;
-            for (var key in configs) {
-                config = configs[key];
-                if ( Lang.hasOwnProperty(configs, key) && 
+            for (var key in this._configs) {
+                config = this._configs[key];
+                if ( Lang.hasOwnProperty(this._configs, key) && 
                         !Lang.isUndefined(config) ) {
                     keys[keys.length] = key;
                 }
@@ -299,9 +299,9 @@ YAHOO.util.Attribute.prototype = {
          * @return {Boolean} Whether or not the value was set
          */
         resetValue: function(key, silent){
-            var configs = this._configs || {};
-            if (configs[key]) {
-                this.set(key, configs[key]._initialConfig.value, silent);
+            this._configs = this._configs || {};
+            if (this._configs[key]) {
+                this.set(key, this._configs[key]._initialConfig.value, silent);
                 return true;
             }
             return false;
@@ -314,17 +314,17 @@ YAHOO.util.Attribute.prototype = {
          * @param {Boolean} silent Whether or not to suppress change events
          */
         refresh: function(key, silent){
-            var configs = this._configs;
+            this._configs = this._configs;
             
             key = ( ( Lang.isString(key) ) ? [key] : key ) || 
                     this.getAttributeKeys();
             
             for (var i = 0, len = key.length; i < len; ++i) { 
                 if ( // only set if there is a value and not null
-                    configs[key[i]] && 
-                    ! Lang.isUndefined(configs[key[i]].value) &&
-                    ! Lang.isNull(configs[key[i]].value) ) {
-                    configs[key[i]].refresh(silent);
+                    this._configs[key[i]] && 
+                    ! Lang.isUndefined(this._configs[key[i]].value) &&
+                    ! Lang.isNull(this._configs[key[i]].value) ) {
+                    this._configs[key[i]].refresh(silent);
                 }
             }
         },
@@ -351,8 +351,8 @@ YAHOO.util.Attribute.prototype = {
          * attribute's properties.
          */
         getAttributeConfig: function(key) {
-            var configs = this._configs || {};
-            var config = configs[key] || {};
+            this._configs = this._configs || {};
+            var config = this._configs[key] || {};
             var map = {}; // returning a copy to prevent overrides
             
             for (key in config) {
@@ -372,13 +372,13 @@ YAHOO.util.Attribute.prototype = {
          * @param {Boolean} init Whether or not this should become the intial config.
          */
         setAttributeConfig: function(key, map, init) {
-            var configs = this._configs || {};
+            this._configs = this._configs || {};
             map = map || {};
-            if (!configs[key]) {
+            if (!this._configs[key]) {
                 map.name = key;
-                configs[key] = new YAHOO.util.Attribute(map, this);
+                this._configs[key] = this.createAttribute(map);
             } else {
-                configs[key].configure(map, init);
+                this._configs[key].configure(map, init);
             }
         },
         
@@ -401,10 +401,30 @@ YAHOO.util.Attribute.prototype = {
          * @private
          */
         resetAttributeConfig: function(key){
-            var configs = this._configs || {};
-            configs[key].resetConfig();
+            this._configs = this._configs || {};
+            this._configs[key].resetConfig();
         },
         
+        // wrapper for EventProvider.subscribe
+        // to create events on the fly
+        subscribe: function(type, callback) {
+            this._events = this._events || {};
+
+            if ( !(type in this._events) ) {
+                this._events[type] = this.createEvent(type);
+            }
+
+            YAHOO.util.EventProvider.prototype.subscribe.apply(this, arguments);
+        },
+
+        on: function() {
+            this.subscribe.apply(this, arguments);
+        },
+
+        addListener: function() {
+            this.subscribe.apply(this, arguments);
+        },
+
         /**
          * Fires the attribute's beforeChange event. 
          * @method fireBeforeChangeEvent
@@ -427,6 +447,10 @@ YAHOO.util.Attribute.prototype = {
         fireChangeEvent: function(e) {
             e.type += 'Change';
             return this.fireEvent(e.type, e);
+        },
+
+        createAttribute: function(map) {
+            return new YAHOO.util.Attribute(map, this);
         }
     };
     
@@ -474,7 +498,7 @@ YAHOO.util.Element.prototype = {
     /**
      * Wrapper for HTMLElement method.
      * @method appendChild
-     * @param {Boolean} deep Whether or not to do a deep clone
+     * @param {YAHOO.util.Element || HTMLElement} child The element to append. 
      */
     appendChild: function(child) {
         child = child.get ? child.get('element') : child;
@@ -572,10 +596,9 @@ YAHOO.util.Element.prototype = {
             }
             
             this.createEvent(type, this);
-            this._events[type] = true;
         }
         
-        this.subscribe.apply(this, arguments); // notify via customEvent
+        YAHOO.util.EventProvider.prototype.subscribe.apply(this, arguments); // notify via customEvent
     },
     
     
@@ -589,6 +612,15 @@ YAHOO.util.Element.prototype = {
      */
     on: function() { this.addListener.apply(this, arguments); },
     
+    /**
+     * Alias for addListener
+     * @method subscribe
+     * @param {String} type The name of the event to listen for
+     * @param {Function} fn The function call when the event fires
+     * @param {Any} obj A variable to pass to the handler
+     * @param {Object} scope The object to use for the scope of the handler 
+     */
+    subscribe: function() { this.addListener.apply(this, arguments); },
     
     /**
      * Remove an event listener
@@ -740,6 +772,23 @@ YAHOO.util.Element.prototype = {
         return AttributeProvider.prototype.get.call(this, key);
     },
 
+    setAttributes: function(map, silent){
+        var el = this.get('element');
+        for (var key in map) {
+            // need to configure if setting unconfigured HTMLElement attribute 
+            if ( !this._configs[key] && !YAHOO.lang.isUndefined(el[key]) ) {
+                this.setAttributeConfig(key);
+            }
+        }
+
+        // set based on configOrder
+        for (var i = 0, len = this._configOrder.length; i < len; ++i) {
+            if (map[this._configOrder[i]]) {
+                this.set(this._configOrder[i], map[this._configOrder[i]], silent);
+            }
+        }
+    },
+
     set: function(key, value, silent) {
         var el = this.get('element');
         if (!el) {
@@ -767,6 +816,7 @@ YAHOO.util.Element.prototype = {
         } else {
             AttributeProvider.prototype.setAttributeConfig.apply(this, arguments);
         }
+        this._configOrder.push(key);
     },
     
     getAttributeKeys: function() {
@@ -782,6 +832,11 @@ YAHOO.util.Element.prototype = {
         
         return keys;
     },
+
+    createEvent: function(type, scope) {
+        this._events[type] = true;
+        AttributeProvider.prototype.createEvent.apply(this, arguments);
+    },
     
     init: function(el, attr) {
         _initElement.apply(this, arguments); 
@@ -792,6 +847,7 @@ var _initElement = function(el, attr) {
     this._queue = this._queue || [];
     this._events = this._events || {};
     this._configs = this._configs || {};
+    this._configOrder = []; 
     attr = attr || {};
     attr.element = attr.element || el || null;
 
@@ -811,53 +867,51 @@ var _initElement = function(el, attr) {
         'submit': true
     };
 
+    var isReady = false;  // to determine when to init HTMLElement and content
+
     if (YAHOO.lang.isString(el) ) { // defer until available/ready
         _registerHTMLAttr.call(this, 'id', { value: attr.element });
     }
 
     if (Dom.get(el)) {
-        _availableHandler.call(this, attr);  
-        _readyHandler.call(this, attr);
-        return; // note return
+        isReady = true;
+        _initHTMLElement.call(this, attr);
+        _initContent.call(this, attr);
     } 
 
     YAHOO.util.Event.onAvailable(attr.element, function() {
-        _availableHandler.call(this, attr);  
+        if (!isReady) { // otherwise already done
+            _initHTMLElement.call(this, attr);
+        }
+
+        this.fireEvent('available', { type: 'available', target: attr.element });  
     }, this, true);
     
     YAHOO.util.Event.onContentReady(attr.element, function() {
-        _readyHandler.call(this, attr);
+        if (!isReady) { // otherwise already done
+            _initContent.call(this, attr);
+        }
+        this.fireEvent('contentReady', { type: 'contentReady', target: attr.element });  
     }, this, true);
 };
 
-var _availableHandler = function(attr) {
-    attr.element = Dom.get(attr.element);
-
+var _initHTMLElement = function(attr) {
     /**
      * The HTMLElement the Element instance refers to.
-     * @config element
+     * @attribute element
      * @type HTMLElement
      */
     this.setAttributeConfig('element', {
-        value: attr.element,
+        value: Dom.get(attr.element),
         readOnly: true
      });
-
-    this.fireEvent('available', {
-        type: 'available',
-        target: attr.element
-    }); 
 };
 
-var _readyHandler = function(attr) {
+var _initContent = function(attr) {
     this.initAttributes(attr);
     this.setAttributes(attr, true);
     this.fireQueue();
 
-    this.fireEvent('contentReady', {
-        type: 'contentReady',
-        target: attr.element
-    });
 };
 
 /**
@@ -910,4 +964,4 @@ var _registerHTMLAttr = function(key, map) {
 YAHOO.augment(YAHOO.util.Element, AttributeProvider);
 })();
 
-YAHOO.register("element", YAHOO.util.Element, {version: "2.2.1", build: "193"});
+YAHOO.register("element", YAHOO.util.Element, {version: "2.4.1", build: "742"});
